# Declares an external package to be used. This macro will declare appropriate 
# command line switches for ./configure and corresponding environment variables
# to define the package's prefix path, library path and include path. The
# configure command line option switches override the corresponding environment
# variables. Various permutations of capitalisation, version number appendage
# and include/lib path will be attempted. Omit the version number if it isn't
# relevant. The third and fourth options will be executed respectively if the
# package is or isn't found: this is an ideal place to place ERROR/WARNING
# messages as appropriate for mandatory/optional packages and to run tests of
# the library if it appears to be there.
#
# TODO: how much overlap with libtool? can we split the lib and include 
# checks into separate macros?
#
#AC_CEDAR_LIBRARYANDHEADERS(PrettyName, ReleaseNumber, action-if-true, action-if-false)
#----------------------------------------
AC_DEFUN([AC_CEDAR_LIBRARYANDHEADERS], [
  ## Define a bunch of case permutations
  m4_define([cedar_PkgName], [$1])dnl
  m4_define([cedar_PKGNAME], [translit([translit([$1], [a-z], [A-Z])], [.])])dnl
  m4_define([cedar_pkgname], [translit([translit([$1], [A-Z], [a-z])], [.])])dnl
  m4_define([cedar_pkgversion], [$2])dnl

  pkgpath=""
  pkglib=no; pkginc=no; pkggood=no

  ## Don't know why this isn't working by default:
  test x${prefix} == xNONE && prefix=${ac_default_prefix}

  ## Environment variables for specifying paths
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@PATH,
    path to cedar_PkgName @<:@$prefix and various standard locations@:>@)
  pkgpath=${cedar_PKGNAME@&t@PATH}

  ## "configure" option switches for specifying paths
  AC_ARG_WITH([cedar_pkgname],
              AC_HELP_STRING(--with-@&t@cedar_pkgname, 
                path to cedar_PkgName @<:@$prefix and various standard locations@:>@),
              [pkgpath=$with_@&t@cedar_pkgname], [])
  if test "$pkgpath"; then cedar_PKGNAME@&t@PATH="$pkgpath"; fi

  ## Has this lib been disabled?
  if test x$pkgpath == xno; then 
    AC_MSG_NOTICE(Not building against cedar_PkgName)
    AM_CONDITIONAL(WITH_@&t@cedar_PKGNAME@&t@INC, false)
    AM_CONDITIONAL(WITH_@&t@cedar_PKGNAME@&t@LIB, false)
  else
    ## Check library and header
    AC_CEDAR_LIBRARY(cedar_PkgName, cedar_pkgversion, , pkglib=no)
    AC_CEDAR_HEADERS(cedar_PkgName, cedar_pkgversion, , pkginc=no)

    ## Execute pass/fail shell code
    if test "x$pkglib" == "xyes" && test "x$pkginc" == "xyes"; then
      #AC_MSG_NOTICE([cedar_PkgName paths verified])
      pkggood="yes"
      $3
    else 
      pkggood="no"
      $4
    fi
  fi

  ## Export variables to automake
  AM_CONDITIONAL(WITH_@&t@cedar_PKGNAME, [test x$pkggood != xno])
])


#AC_CEDAR_HEADERS(PrettyName, ReleaseNumber, action-if-true, action-if-false)
AC_DEFUN([AC_CEDAR_HEADERS], [
  ## Define a bunch of case permutations
  m4_define([cedar_PkgName], [$1])dnl
  m4_define([cedar_PKGNAME], [translit([translit([$1], [a-z], [A-Z])], [.])])dnl
  m4_define([cedar_pkgname], [translit([translit([$1], [A-Z], [a-z])], [.])])dnl
  m4_define([cedar_IncName], [cedar_PkgName])dnl
  m4_define([cedar_INCNAME], [cedar_PKGNAME])dnl
  m4_define([cedar_incname], [cedar_pkgname])dnl
  m4_define([cedar_pkgversion], [$2])dnl

  ## We have a set of user-set variables:
  pkgpath=""; pkgincpath=""
  ## Also need some status variables:
  pkginc=no

  ## Don't know why this isn't working by default:
  test x${prefix} == xNONE && prefix=${ac_default_prefix}

  ## Environment variables for specifying paths
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@PATH,
    path to cedar_PkgName @<:@$prefix and various standard locations@:>@)
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@INCPATH,
    path to the directory containing the cedar_PkgName header files @<:@cedar_PKGNAME@&t@PATH/include/cedar_PkgName@:>@)
  pkgpath=${cedar_PKGNAME@&t@PATH}
  pkgincpath=${cedar_PKGNAME@&t@INCPATH}

  ## "configure" option switches for specifying paths
  AC_ARG_WITH([cedar_pkgname],
              AC_HELP_STRING(--with-@&t@cedar_pkgname, 
                path to cedar_PkgName @<:@$prefix and various standard locations@:>@),
              [pkgpath=$with_@&t@cedar_pkgname], [])
  AC_ARG_WITH(cedar_pkgname@&t@-incpath,
              AC_HELP_STRING(--with-@&t@cedar_pkgname@&t@-incpath, 
                path to directory containing cedar_PkgName headers @<:@cedar_PKGNAME@&t@PATH/include/cedar_PkgName@:>@),
              [pkgincpath=$with_@&t@cedar_pkgname@&t@_incpath], [])
  if test "$pkgpath"; then cedar_PKGNAME@&t@PATH="$pkgpath"; fi

  ## Has this header been disabled?
  if test x$pkgpath == xno; then 
    AC_MSG_NOTICE(Not building against cedar_PkgName)
  else
    ## Base paths
    pkgbases="$prefix $ac_default_prefix /usr /"
    if test "$pkgpath"; then pkgbases="$pkgpath"; fi

    ## Look for include files: first build the search list...
    incpaths=""
    for base in $pkgbases; do incpaths="$incpaths $base/include/cedar_IncName $base/include/cedar_incname"; done
    if test "$pkgincpath"; then incpaths="$pkgincpath"; fi
    ## .. and then do the search:
    for incpath in $incpaths; do
      incpath=`echo $incpath | sed -e s:'//*':'/':g -e s:'/$':'':`
      if test -d $incpath; then
        pkginc=yes; break; else pkginc=no;
      fi
      if test x$pkginc != xno; then break; fi
    done
    if test x$pkginc != xno; then
      if test `basename $incpath` == "cedar_PkgName" -o \
              `basename $incpath` == "cedar_PKGNAME" -o \
              `basename $incpath` == "cedar_pkgname"; then
        cedar_PKGNAME@&t@INCPATH=`dirname $incpath | sed -e s:'/$':'':`
        cedar_PKGNAME@&t@INCNAME=`basename $incpath`
      else
        cedar_PKGNAME@&t@INCPATH=$incpath
        cedar_PKGNAME@&t@INCNAME=""
      fi
      AC_MSG_NOTICE([Found cedar_PkgName header directory at $incpath])
    else
      AC_MSG_WARN(cedar_PkgName header directory was not found)
    fi

    ## Execute pass/fail shell code
    if test "x$pkginc" == "xyes"; then
      true
      $3
    else 
      true
      $4
    fi
  fi

  ## Export variables to automake
  AC_SUBST(cedar_PKGNAME@&t@INCPATH)
  AC_SUBST(cedar_PKGNAME@&t@INCNAME)
  AM_CONDITIONAL(WITH_@&t@cedar_PKGNAME@&t@INC, [test x$pkgginc != xno])
])


#AC_CEDAR_LIBRARY(PrettyName, ReleaseNumber, action-if-true, action-if-false)
AC_DEFUN([AC_CEDAR_LIBRARY], [
  ## Define a bunch of case permutations
  m4_define([cedar_PkgName], [$1])dnl
  m4_define([cedar_PKGNAME], [translit([translit([$1], [a-z], [A-Z])], [.])])dnl
  m4_define([cedar_pkgname], [translit([translit([$1], [A-Z], [a-z])], [.])])dnl
  m4_define([cedar_LibName], [cedar_PkgName])dnl
  m4_define([cedar_LIBNAME], [cedar_PKGNAME])dnl
  m4_define([cedar_libname], [cedar_pkgname])dnl
  m4_define([cedar_libversion], [$2])dnl

  ## We have a set of user-set variables:
  pkgpath=""; pkglibpath=""; pkglibname=""
  ## Also need some status variables:
  pkglib=no

  ## Don't know why this isn't working by default:
  test x${prefix} == xNONE && prefix=${ac_default_prefix}

  ## Environment variables for specifying paths
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@PATH,
    path to cedar_PkgName @<:@$prefix and various standard locations@:>@)
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@LIBPATH,
    path to the directory containing the cedar_PkgName library @<:@cedar_PKGNAME@&t@PATH/lib or cedar_PKGNAME@&t@PATH/lib/cedar_PkgName@:>@)
  AC_ARG_VAR(@&t@cedar_PKGNAME@&t@LIBNAME,
    name to be used when linking the cedar_PkgName library @<:@cedar_PkgName@:>@)
  pkgpath=${cedar_PKGNAME@&t@PATH}
  pkglibpath=${cedar_PKGNAME@&t@LIBPATH}
  pkglibname=${cedar_PKGNAME@&t@LIBFLAG}

  ## "configure" option switches for specifying paths
  AC_ARG_WITH([cedar_pkgname],
              AC_HELP_STRING(--with-@&t@cedar_pkgname, 
                path to cedar_PkgName @<:@$prefix and various standard locations@:>@),
              [pkgpath=$with_@&t@cedar_pkgname], [])
  AC_ARG_WITH(cedar_pkgname@&t@-ldpath,
              AC_HELP_STRING(--with-@&t@cedar_pkgname@&t@-libpath, 
                path to directory containing cedar_PkgName library @<:@cedar_PKGNAME@&t@PATH/lib or cedar_PKGNAME@&t@PATH/lib/cedar_PkgName@:>@),
              [pkglibpath=$with_@&t@cedar_pkgname@&t@_libpath], [])
  AC_ARG_WITH(cedar_pkgname@&t@-libname,
              AC_HELP_STRING(--with-@&t@cedar_pkgname@&t@-libname,
                name to be used when linking the cedar_PkgName library @<:@cedar_PkgName@:>@),
              [pkglibname=$with_@&t@cedar_pkgname@&t@_libname], [])
  if test "$pkgpath"; then cedar_PKGNAME@&t@PATH="$pkgpath"; fi

  ## Has this lib been disabled?
  if test x$pkgpath == xno; then 
    AC_MSG_NOTICE(Not building against cedar_PkgName)
  else
    ## Base paths
    pkgbases="$prefix $ac_default_prefix /usr / ./"
    if test "$pkgpath"; then pkgbases="$pkgpath"; fi

    ## Build a list of library search locations, unless specified
    libpaths=""
    for base in $pkgbases; do 
      libpaths="$libpaths $base/lib $base/lib/cedar_PkgName $base/lib/cedar_pkgname"
    done
    libpaths="$libpaths ./src"
    if test "$pkglibpath"; then libpaths="$pkglibpath"; fi
    ## Look for library with various name permutations  
    for libpath in $libpaths; do
      libpath=`echo $libpath | sed -e s:'//*':'/':g -e s:'/$':'':`
      libnames="cedar_LibName cedar_libname cedar_LIBNAME"
      if test "$pkglibname"; then libnames=$pkglibname; fi
      libversions="-cedar_libversion cedar_libversion"
      for libversion in $libversions ""; do
        for libname in $libnames; do
          for libextn in so la dylib a; do
            #echo "Testing ${libpath}/lib${libname}${libversion}.${libextn}"
            if test -e ${libpath}/lib${libname}${libversion}.${libextn}; then 
              pkglib=yes; break; else pkglib=no; 
            fi
          done
          if test x$pkglib != xno; then break; fi
        done
        if test x$pkglib != xno; then break; fi
      done
      if test x$pkglib != xno; then break; fi
    done
    ## Announce success/failure and set variables
    if test x$pkglib != xno; then
      libfullpath=`echo ${libpath}/lib${libname}${libversion}.${libextn}`
      cedar_PKGNAME@&t@LIBPATH=`dirname $libfullpath`
      #`echo $libpath | sed -e s:'//*':'/':g -e s:'/$':'':`
      cedar_PKGNAME@&t@LIBNAME=`basename $libfullpath | sed -e s/'^lib'// -e s/'\.@<:@a-zA-Z@:>@*$'//`
      #echo :$cedar_PKGNAME@&t@LIBNAME:
      AC_MSG_NOTICE([Found cedar_PkgName library at ${libfullpath}])
    else
      AC_MSG_WARN(cedar_PkgName library was not found)
    fi

    ## Execute pass/fail shell code
    if test "x$pkglib" == "xyes"; then
      true
      $3
    else 
      true
      $4
    fi
  fi

  ## Export variables to automake
  AC_SUBST(cedar_PKGNAME@&t@LIBPATH)
  AC_SUBST(cedar_PKGNAME@&t@LIBNAME)
  AM_CONDITIONAL(WITH_@&t@cedar_PKGNAME@&t@LIB, [test x$pkglib != xno])
])


# Create a variable called EMPTY with appropriate content
#AC_EMPTY_SUBST
AC_DEFUN([AC_EMPTY_SUBST],
[EMPTY=""
AC_SUBST(EMPTY)
])
